#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#include <linux/module.h>

#include "nikal.h"

#ifdef nNIKAL100_kCopyrightString
   MODULE_LICENSE(nNIKAL100_kCopyrightString);
#endif

#ifdef nNIKAL100_kAuthorString
   MODULE_AUTHOR(nNIKAL100_kAuthorString);
#endif

#ifdef nNIKAL100_kDescriptionString
   MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
#endif


#include <linux/pci.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static struct pci_device_id nipciidtable[] = {
   {0x1093, 0xC4C4, 0x1093, 0x7755, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x774B, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7841, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7735, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x774D, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77DB, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77DC, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7840, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77B9, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77DD, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7875, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7878, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E3, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79D0, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E8, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E5, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E4, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79CE, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E9, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78B7, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78E7, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78EA, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79DD, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79DE, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A2B, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79DF, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x79E0, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A2A, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A38, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A39, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A41, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A3A, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A43, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A3B, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7A5C, 0, 0, (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(pci, nipciidtable);

static struct pci_driver niPciDriver = {
   .name = "atomicchinchk",
   .id_table = nipciidtable,
};

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL200_registerPCIDriver(&niPciDriver);
}
void unregisterNativeDriver(void* data)
{
   nNIKAL200_unregisterPCIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
